function initBots()
{
	bots.get('mute').active = true;
	bots.get('autociv').active = true;
	bots.get('link').active = true;

	bots.get('autociv').civList = g_PlayerCivList;
	bots.get('master').init();
}

let game = {
	'set':
	{
		'resources': (quantity) =>
		{
			if (!g_IsController) return;
			let val = parseInt(quantity);
			if (!Number.isInteger(val)) return addChatSystemMessage('Invalid starting resources value (must be a number).');
			g_GameAttributes.settings.StartingResources = val;
			Engine.SendNetworkChat(`Starting resources set to: ${val}`);
			updateGameAttributes();
		},
		'population': (quantity) =>
		{
			if (!g_IsController) return;
			let val = parseInt(quantity);
			if (!Number.isInteger(val) || val < 0) return addChatSystemMessage('Invalid population cap value (must be a number >= 0).');
			g_GameAttributes.settings.PopulationCap = val;
			Engine.SendNetworkChat(`Population cap set to: ${val}`);
			updateGameAttributes();
		},
		'mapsize': (mapsize) =>
		{
			if (!g_IsController) return;
			let val = parseInt(mapsize);
			if (!Number.isInteger(val) || val < 1) return addChatSystemMessage('Invalid map size value (must be a number >= 1).');
			g_GameAttributes.settings.Size = val;
			Engine.SendNetworkChat(`Map size set to: ${val}`);
			updateGameAttributes();
		},
		'player':
		{
			'civ': (playerName, playerCivCode) =>
			{
				let playerId, playerPos, civCodeIndex;
				if ((playerId = game.get.player.id(playerName)) == undefined) return;
				if ((playerPos = game.get.player.pos(playerId)) == -1) return;
				if ((civCodeIndex = g_PlayerCivList.code.indexOf(playerCivCode)) == -1) return;
				g_PlayerDropdowns.playerCiv.select(civCodeIndex, playerPos - 1);
				updateGameAttributes();
			},
			'observer': (playerName) =>
			{
				let playerId, playerPos;
				if ((playerId = game.get.player.id(playerName)) == undefined) return;
				if ((playerPos = game.get.player.pos(playerId)) == -1) return;
				Engine.AssignNetworkPlayer(playerPos, '');
			}
		}
	},
	'get':
	{
		'player':
		{
			// Returns undefined if no player with that name
			'id': playerName => Object.keys(g_PlayerAssignments).find(id => g_PlayerAssignments[id].name == playerName),
			// Returns -1 in case of observer, undefined if Id does not exist
			'pos': playerId => g_PlayerAssignments[playerId].player,
			'selfName': () => g_PlayerAssignments[Engine.GetPlayerGUID()].name,
		},
		'players':
		{
			'name': () => Object.keys(g_PlayerAssignments).map(id => g_PlayerAssignments[id].name)
		}
	}
}

function botToggleAction(botName)
{
	let bottyMcBotFace = bots.get(botName);
	bottyMcBotFace.toggle();
	addChatSystemMessage(`${bottyMcBotFace.name} ${bottyMcBotFace.active ? 'activated' : 'deactivated'}.`);
}

g_NetworkCommands['/help'] = () => addChatSystemMessage(Object.keys(g_NetworkCommands).join(', '));

g_NetworkCommands['/mutelist'] = () => addChatSystemMessage(bots.get('mute').instance.getInfo());
g_NetworkCommands['/muteclear'] = () => bots.get('mute').instance.clear();
g_NetworkCommands['/mute'] = player =>
{
	bots.get('mute').instance.mute(player);
	Engine.SendNetworkChat(`${game.get.player.selfName()} has muted ${player}.`);
};
g_NetworkCommands['/unmute'] = player =>
{
	bots.get('mute').instance.unmute(player);
	Engine.SendNetworkChat(`${game.get.player.selfName()} has unmuted ${player}.`)
};

g_NetworkCommands['/resources'] = value => game.set.resources(value);
g_NetworkCommands['/population'] = value => game.set.population(value);
g_NetworkCommands['/mapsize'] = size => game.set.mapsize(size);

g_NetworkCommands['/autociv'] = () => { if (g_IsController) botToggleAction('autociv'); };
g_NetworkCommands['/nub'] = () => { if (g_IsController) botToggleAction('nub'); };
g_NetworkCommands['/noproblemo'] = () => { if (g_IsController) botToggleAction('noproblemo'); };

g_NetworkCommands['/linklist'] = () => addChatSystemMessage(bots.get('link').instance.getInfo());
g_NetworkCommands['/link'] = index =>
{
	let err = chatLinks.openLink(index);
	if (err) addChatSystemMessage(err);
}

g_NetworkCommands['/ready'] = () => { if (!g_IsController) toggleReady(); };
g_NetworkCommands['/start'] = () => { if (g_IsController) launchGame(); };
