addChatMessage = (function (originalFunction)
{
	return function (msg)
	{
		if (msg.type === "chat")
		{
			let data = Object.assign({},{
				"receiver": g_PlayerAssignments[Engine.GetPlayerGUID()].name,
				"sender": g_PlayerAssignments[msg.guid].name,
				"message": msg.text
			});

			if (bots.get("master").msgReact(data))
				return;
		}
		originalFunction(msg);
	}
})(addChatMessage);

// compatibility with fgod mod
if (typeof (getFilteredMods) != "function")
{
	// var makes it global scope
	var getFilteredMods = function ()
	{
		return Engine.GetEngineInfo().mods.filter(mod => !mod[0].startsWith("fgod"));
	};
}

getFilteredMods = (function (originalFunction)
{
	return function ()
	{
		const modName = "autociv"; // mod to ignore
		return originalFunction().filter(mod => !mod[0].startsWith(modName));
	}
})(getFilteredMods);

sendRegisterGameStanzaImmediate = function ()
{
	if (!g_IsController || !Engine.HasXmppClient()) return;

	if (g_GameStanzaTimer !== undefined)
	{
		clearTimeout(g_GameStanzaTimer);
		g_GameStanzaTimer = undefined;
	}

	let clients = formatClientsForStanza();
	let stanza = {
		"name": g_ServerName,
		"port": g_ServerPort,
		"hostUsername": Engine.LobbyGetNick(),
		"mapName": g_GameAttributes.map,
		"niceMapName": getMapDisplayName(g_GameAttributes.map),
		"mapSize": g_GameAttributes.mapType == "random" ? g_GameAttributes.settings.Size : "Default",
		"mapType": g_GameAttributes.mapType,
		"victoryConditions": g_GameAttributes.settings.VictoryConditions.join(","),
		"nbp": clients.connectedPlayers,
		"maxnbp": g_GameAttributes.settings.PlayerData.length,
		"players": clients.list,
		"stunIP": g_StunEndpoint ? g_StunEndpoint.ip : "",
		"stunPort": g_StunEndpoint ? g_StunEndpoint.port : "",
		"mods": JSON.stringify(getFilteredMods())
	};

	// Only send the stanza if the relevant settings actually changed
	if (g_LastGameStanza && Object.keys(stanza).every(prop => g_LastGameStanza[prop] == stanza[prop]))
		return;

	g_LastGameStanza = stanza;
	Engine.SendRegisterGame(stanza);
};

function reRegister()
{
	if (!g_IsController) return;
	if (Engine.IsXmppClientConnected()) return;

	Engine.ConnectXmppClient()
	setTimeout(reconnect_sendRegisterGameStanzaImmediate, 1000)
	return;
}

function reconnect_sendRegisterGameStanzaImmediate()
{
	if (!g_IsController) return;

	if (g_GameStanzaTimer !== undefined)
	{
		clearTimeout(g_GameStanzaTimer);
		g_GameStanzaTimer = undefined;
	}

	let clients = formatClientsForStanza();
	let stanza = {
		"name": g_ServerName,
		"port": g_ServerPort,
		"hostUsername": Engine.LobbyGetNick(),
		"mapName": g_GameAttributes.map,
		"niceMapName": getMapDisplayName(g_GameAttributes.map),
		"mapSize": g_GameAttributes.mapType == "random" ? g_GameAttributes.settings.Size : "Default",
		"mapType": g_GameAttributes.mapType,
		"victoryConditions": g_GameAttributes.settings.VictoryConditions.join(","),
		"nbp": clients.connectedPlayers,
		"maxnbp": g_GameAttributes.settings.PlayerData.length,
		"players": clients.list,
		"stunIP": g_StunEndpoint ? g_StunEndpoint.ip : "",
		"stunPort": g_StunEndpoint ? g_StunEndpoint.port : "",
		"mods": JSON.stringify(getFilteredMods())
	};

	g_LastGameStanza = stanza;
	Engine.SendRegisterGame(stanza);
}

function setGameName(name)
{
	if (!g_IsController) return;
	g_ServerName = name;
	reconnect_sendRegisterGameStanzaImmediate();
}

// set focus on set input when hotkey="cancel" is activated and arg is undefined
selectPanel = (function (originalFunction)
{
	return function (arg)
	{
		originalFunction(arg);
		if (arg == undefined)
			Engine.GetGUIObjectByName("chatInput").focus();
	}
})(selectPanel);

init = (function (originalFunction)
{
	return function (args)
	{
		initBots();
		originalFunction(args);
	}
})(init);