addChatMessage = (function (originalFunction)
{
	return function (msg)
	{
		if (!!msg.from && msg.from !== "system")
		{
			let data = Object.assign({}, {
				"receiver": Engine.LobbyGetNick(),
				"sender": msg.from,
				"message": msg.text
			});

			if (bots.get("master").msgReact(data))
				return;
		}
		originalFunction(msg);
	};
})(addChatMessage)

function initBots()
{
	bots.get('mute').active = true;
	bots.get('link').active = true;
	bots.get('master').init();
}

function addSpecialChatSystemMessage(text)
{
	addChatMessage(
		{
			"type": "system",
			"isSpecial": true,
			"text": "/special " + text
		});
}

g_ChatCommands["pingall"] = {
	"description": translate("Ping all 'Online' and 'Observer' players."),
	"handler": function (args)
	{
		let selfNick = Engine.LobbyGetNick();
		let ignore = new Set([selfNick, "Ratings", "WFGbot", "Triumvir", "user1"]);
		let candidatesToAnnoy = new Set();

		for (let host of g_GameList)
		{
			let players = stringifiedTeamListToPlayerData(host.players);
			let selfInHost = players.some(player => splitRatingFromNick(player.Name).nick == selfNick);
			for (let player of players)
				if (selfInHost)
					ignore.add(splitRatingFromNick(player.Name).nick);
				else if (player.Team == "observer")
					candidatesToAnnoy.add(splitRatingFromNick(player.Name).nick);
		}

		for (let player of Engine.GetPlayerList())
			if (player.presence == "available")
				candidatesToAnnoy.add(player.name);

		let annoyList = Array.from(candidatesToAnnoy.difference(ignore)).join(", ");
		Engine.LobbySendMessage(annoyList);
		if (!!args[0].trim())
			Engine.LobbySendMessage(args.join(" "))

		return false;
	}
};

g_ChatCommands["playing"] = {
	"description": translate("Set your state to 'Playing'."),
	"handler": function ()
	{
		Engine.LobbySetPlayerPresence("playing");
		return false;
	}
};


// Local mute commands -----------------------------------------------
g_ChatCommands["mute"] = {
	"description": "Mute player.",
	"handler": function (args)
	{
		bots.get("mute").instance.mute(args[0]);
		addSpecialChatSystemMessage(`You have muted ${args[0]}`);
		return false;
	}
};

g_ChatCommands["unmute"] = {
	"description": "Unmute player.",
	"handler": function (args)
	{
		bots.get("mute").instance.unmute(args[0]);
		addSpecialChatSystemMessage(`You have unmuted ${args[0]}`);
		return false;
	}
};

g_ChatCommands["muteclear"] = {
	"description": "Clear list of muted players.",
	"handler": function ()
	{
		bots.get("mute").instance.clear();
		addSpecialChatSystemMessage("You have cleared lobby muted list");
		return false;
	}
};

g_ChatCommands["mutelist"] = {
	"description": "List of muted players.",
	"handler": function ()
	{
		addSpecialChatSystemMessage(bots.get("mute").instance.getInfo());
		return false;
	}
};

// Link commands -----------------------------------------------------
g_ChatCommands["linklist"] = {
	"description": "Shows the list of links detected in the chat.",
	"handler": function ()
	{
		addSpecialChatSystemMessage(bots.get("link").instance.getInfo());
		return false;
	}
};

g_ChatCommands["link"] = {
	"description": "Open last link or older link of index is writte.",
	"handler": function (args)
	{
		let err = bots.get("link").instance.openLink(args[0]);
		if (err)
			addSpecialChatSystemMessage(err);
		return false;
	}
};

g_NetMessageTypes["chat"]["subject"] = msg =>
{
	// don't add subject kilometric message
	updateSubject(msg.subject);
	return false;
};

function handleInputBeforeGui(ev)
{
	resizeBar.onEvent(ev);
	return false;
}

init = (function (originalFunction)
{
	return function (...args)
	{
		initBots();

		originalFunction(...args);

		let fgodMod = Engine.GetEngineInfo().mods.some(mod => mod[0].startsWith("fgod"));

		let hookList = [
			["profilePanel", "right"],
			["leftButtonPanel", "right"],
			[fgodMod ? "playerList" : "playersBox", "right"]
		];
		if (fgodMod)
			hookList.push(["presenceDropdown", "right"], ["playerGamesNumber", "right"])

		resizeBar("chatPanel", "top", undefined, [[fgodMod ? "gameList" : "gamesBox", "bottom"]])
		resizeBar("middlePanel", "left", undefined, hookList);
		resizeBar("rightPanel", "left", undefined, [["middlePanel", "right"]]);

		let gameInfo = Engine.GetGUIObjectByName("gameInfo");
		let gameInfoUsers = gameInfo.children[gameInfo.children.length - 1];
		let gameInfoDescription = gameInfo.children[gameInfo.children.length - 2];
		resizeBar(gameInfoUsers, "top", undefined, [[gameInfoDescription, "bottom"]], () => !gameInfo.hidden);
	}
})(init);

onTick = (function (originalFunction)
{
	return function ()
	{
		originalFunction();
		resizeBar.onTick();
		animate.onTick();
	}
})(onTick);
